@@ -32,6 +32,7 @@
 #include <libxml/xpathInternals.h>
 #include <libxml/parserInternals.h>
 #include <libxml/uri.h>
+#include <libxml/xpointer.h>
 #include "xslt.h"
 #include "xsltInternals.h"
 #include "xsltutils.h"
@@ -97,6 +98,112 @@ xsltXPathFunctionLookup (xmlXPathContextPtr ctxt,
  *									*
  ************************************************************************/
 
+static void
+xsltDocumentFunctionLoadDocument(xmlXPathParserContextPtr ctxt, xmlChar* URI)
+{
+    xsltTransformContextPtr tctxt;
+    xmlURIPtr uri;
+    xmlChar *fragment;
+    xsltDocumentPtr xsltdoc;
+    xmlDocPtr doc;
+    xmlXPathContextPtr xptrctxt;
+    xmlXPathObjectPtr object;
+
+    tctxt = xsltXPathGetTransformContext(ctxt);
+    if (tctxt == NULL) {
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+			   "document() : internal error tctxt == NULL\n");
+	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+	return;
+    } 
+	
+    uri = xmlParseURI((const char *) URI);
+    if (uri == NULL) {
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+			   "document() : failed to parse URI\n");
+	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+	return;
+    } 
+    
+    /*
+     * check for and remove fragment identifier
+     */
+    fragment = uri->fragment;
+    if (fragment != NULL) {
+	uri->fragment = NULL;
+	URI = xmlSaveUri(uri);
+	xsltdoc = xsltLoadDocument(tctxt, URI);
+	xmlFree(URI);
+    } else
+	xsltdoc = xsltLoadDocument(tctxt, URI);
+    xmlFreeURI(uri);
+    
+    if (xsltdoc == NULL) {
+	if ((URI == NULL) ||
+	    (URI[0] = '#') ||
+	    (xmlStrEqual(tctxt->style->doc->URL, URI))) {
+	    doc = tctxt->style->doc;
+	} else {
+	    valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+
+	    if (fragment != NULL)
+		xmlFree(fragment);
+
+	    return;
+	}
+    }
+    doc = xsltdoc->doc;
+
+    if ( fragment == NULL ) {
+	valuePush(ctxt,
+		  xmlXPathNewNodeSet((xmlNodePtr) doc));
+	return;
+    }
+	
+    /* use XPointer of HTML location for fragment ID */
+    
+    xptrctxt = xmlXPtrNewContext(doc, NULL, NULL);
+    if (xptrctxt == NULL) {
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+			   "document() : internal error xptrctxt == NULL\n");
+	goto out_fragment;
+    }
+
+    object = xmlXPtrEval(fragment, xptrctxt);
+    xmlFree(fragment);
+    xmlXPathFreeContext(xptrctxt);
+
+    if (object == NULL)
+	goto out_fragment;
+	
+    switch (object->type) {
+    case XPATH_NODESET:
+	break;
+    case XPATH_UNDEFINED:
+    case XPATH_BOOLEAN:
+    case XPATH_NUMBER:
+    case XPATH_STRING:
+    case XPATH_POINT:
+    case XPATH_USERS:
+    case XPATH_XSLT_TREE:
+    case XPATH_RANGE:
+    case XPATH_LOCATIONSET:
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+			   "document() : XPointer does not select a node set: #%s\n", 
+			   fragment);
+	goto out_object;
+    }
+    
+    valuePush(ctxt, object);
+    return;
+
+out_object:
+    xmlXPathFreeObject(object);
+
+out_fragment:
+    valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+}
+
 /**
  * xsltDocumentFunction:
  * @ctxt:  the XPath Parser context
@@ -108,24 +215,19 @@ xsltXPathFunctionLookup (xmlXPathContextPtr ctxt,
 void
 xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
 {
-    xsltDocumentPtr doc;
     xmlXPathObjectPtr obj, obj2 = NULL;
     xmlChar *base = NULL, *URI;
 
 
     if ((nargs < 1) || (nargs > 2)) {
-        xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL,
-                              NULL);
-        xsltGenericError(xsltGenericErrorContext,
+        xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
                          "document() : invalid number of args %d\n",
                          nargs);
         ctxt->error = XPATH_INVALID_ARITY;
         return;
     }
     if (ctxt->value == NULL) {
-        xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL,
-                              NULL);
-        xsltGenericError(xsltGenericErrorContext,
+        xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
                          "document() : invalid arg value\n");
         ctxt->error = XPATH_INVALID_TYPE;
         return;
@@ -133,9 +235,7 @@ xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
 
     if (nargs == 2) {
         if (ctxt->value->type != XPATH_NODESET) {
-            xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-                                  NULL, NULL);
-            xsltGenericError(xsltGenericErrorContext,
+            xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
                              "document() : invalid arg expecting a nodeset\n");
             ctxt->error = XPATH_INVALID_TYPE;
             return;
@@ -182,9 +282,7 @@ xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
      */
     xmlXPathStringFunction(ctxt, 1);
     if (ctxt->value->type != XPATH_STRING) {
-        xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL,
-                              NULL);
-        xsltGenericError(xsltGenericErrorContext,
+        xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
                          "document() : invalid arg expecting a string\n");
         ctxt->error = XPATH_INVALID_TYPE;
         if (obj2 != NULL)
@@ -223,34 +321,9 @@ xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
         if (URI == NULL) {
             valuePush(ctxt, xmlXPathNewNodeSet(NULL));
         } else {
-            xsltTransformContextPtr tctxt;
-
-            tctxt = xsltXPathGetTransformContext(ctxt);
-            if (tctxt == NULL) {
-                xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-                                      NULL, NULL);
-                xsltGenericError(xsltGenericErrorContext,
-                                 "document() : internal error tctxt == NULL\n");
-                valuePush(ctxt, xmlXPathNewNodeSet(NULL));
-            } else {
-                doc = xsltLoadDocument(tctxt, URI);
-                if (doc == NULL) {
-		    if ((xmlStrEqual(URI, BAD_CAST "")) ||
-			(xmlStrEqual(tctxt->style->doc->URL, URI))) {
-			valuePush(ctxt, xmlXPathNewNodeSet(
-				    (xmlNodePtr)tctxt->style->doc));
-		    } else {
-			valuePush(ctxt, xmlXPathNewNodeSet(NULL));
-		    }
-		} else {
-                    /* TODO: use XPointer of HTML location for fragment ID */
-                    /* pbm #xxx can lead to location sets, not nodesets :-) */
-                    valuePush(ctxt,
-                              xmlXPathNewNodeSet((xmlNodePtr) doc->doc));
-                }
-            }
-            xmlFree(URI);
-        }
+	    xsltDocumentFunctionLoadDocument( ctxt, URI );
+	    xmlFree(URI);
+	}
     }
     xmlXPathFreeObject(obj);
     if (obj2 != NULL)
@@ -274,18 +347,17 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
     xsltTransformContextPtr tctxt;
 
     if (nargs != 2) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"key() : expects two arguments\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
 
     obj2 = valuePop(ctxt);
+    xmlXPathStringFunction(ctxt, 1);
     if ((obj2 == NULL) ||
 	(ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "key() : invalid arg expecting a string\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	xmlXPathFreeObject(obj2);
@@ -294,7 +366,7 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
     }
     obj1 = valuePop(ctxt);
 
-    if (obj2->type == XPATH_NODESET) {
+    if ((obj2->type == XPATH_NODESET) || (obj2->type == XPATH_XSLT_TREE)) {
 	int i;
 	xmlXPathObjectPtr newobj, ret;
 
@@ -331,9 +403,7 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	    if (prefix != NULL) {
 		keyURI = xmlXPathNsLookup(ctxt->context, prefix);
 		if (keyURI == NULL) {
-		    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-					  NULL, NULL);
-		    xsltGenericError(xsltGenericErrorContext,
+		    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 			"key() : prefix %s is not bound\n", prefix);
 		}
 		xmlFree(prefix);
@@ -348,9 +418,7 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	valuePush(ctxt, obj2);
 	xmlXPathStringFunction(ctxt, 1);
 	if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-				  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"key() : invalid arg expecting a string\n");
 	    ctxt->error = XPATH_INVALID_TYPE;
 	    xmlXPathFreeObject(obj1);
@@ -498,9 +566,7 @@ xsltGenerateIdFunction(xmlXPathParserContextPtr ctxt, int nargs){
 
 	if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_NODESET)) {
 	    ctxt->error = XPATH_INVALID_TYPE;
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-				  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"generate-id() : invalid arg expecting a node-set\n");
 	    return;
 	}
@@ -519,8 +585,7 @@ xsltGenerateIdFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	}
 	xmlXPathFreeObject(obj);
     } else {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"generate-id() : invalid number of args %d\n", nargs);
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
@@ -550,15 +615,13 @@ xsltSystemPropertyFunction(xmlXPathParserContextPtr ctxt, int nargs){
     const xmlChar *nsURI = NULL;
 
     if (nargs != 1) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"system-property() : expects one string arg\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
     if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "system-property() : invalid arg expecting a string\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	return;
@@ -573,9 +636,7 @@ xsltSystemPropertyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	} else {
 	    nsURI = xmlXPathNsLookup(ctxt->context, prefix);
 	    if (nsURI == NULL) {
-		xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-			              NULL, NULL);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		    "system-property() : prefix %s is not bound\n", prefix);
 	    }
 	}
@@ -647,15 +708,13 @@ xsltElementAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     xsltTransformContextPtr tctxt;
 
     if (nargs != 1) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"element-available() : expects one string arg\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
     if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "element-available() : invalid arg expecting a string\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	return;
@@ -663,8 +722,7 @@ xsltElementAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     obj = valuePop(ctxt);
     tctxt = xsltXPathGetTransformContext(ctxt);
     if (tctxt == NULL) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"element-available() : internal error tctxt == NULL\n");
 	xmlXPathFreeObject(obj);
 	valuePush(ctxt, xmlXPathNewBoolean(0));
@@ -682,9 +740,7 @@ xsltElementAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     } else {
 	nsURI = xmlXPathNsLookup(ctxt->context, prefix);
 	if (nsURI == NULL) {
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-				  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"element-available() : prefix %s is not bound\n", prefix);
 	}
     }
@@ -717,15 +773,13 @@ xsltFunctionAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     const xmlChar *nsURI = NULL;
 
     if (nargs != 1) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"function-available() : expects one string arg\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
     if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "function-available() : invalid arg expecting a string\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	return;
@@ -738,9 +792,7 @@ xsltFunctionAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     } else {
 	nsURI = xmlXPathNsLookup(ctxt->context, prefix);
 	if (nsURI == NULL) {
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-				  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"function-available() : prefix %s is not bound\n", prefix);
 	}
     }
@@ -771,16 +823,14 @@ xsltCurrentFunction(xmlXPathParserContextPtr ctxt, int nargs){
     xsltTransformContextPtr tctxt;
 
     if (nargs != 0) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"current() : function uses no argument\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
     tctxt = xsltXPathGetTransformContext(ctxt);
     if (tctxt == NULL) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"current() : internal error tctxt == NULL\n");
 	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
     } else {
