@@ -9,6 +9,7 @@
  * daniel@veillard.com
  */
 
+#define IN_LIBXSLT
 #include "libxslt.h"
 
 #include <string.h>
@@ -52,8 +53,7 @@ xsltNewStackElem(void) {
 
     cur = (xsltStackElemPtr) xmlMalloc(sizeof(xsltStackElem));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltNewStackElem : malloc failed\n");
 	return(NULL);
     }
@@ -81,8 +81,7 @@ xsltCopyStackElem(xsltStackElemPtr elem) {
 
     cur = (xsltStackElemPtr) xmlMalloc(sizeof(xsltStackElem));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltCopyStackElem : malloc failed\n");
 	return(NULL);
     }
@@ -143,7 +142,8 @@ xsltFreeStackElemList(xsltStackElemPtr elem) {
  *
  * check wether the variable or param is already defined
  *
- * Returns 1 if present, 0 if not, -1 in case of failure.
+ * Returns 1 if variable is present, 2 if param is present, 3 if this
+ *         is an inherited param, 0 if not found, -1 in case of failure.
  */
 static int
 xsltCheckStackElem(xsltTransformContextPtr ctxt, const xmlChar *name,
@@ -159,6 +159,12 @@ xsltCheckStackElem(xsltTransformContextPtr ctxt, const xmlChar *name,
 	    if (((nameURI == NULL) && (cur->nameURI == NULL)) ||
 		((nameURI != NULL) && (cur->nameURI != NULL) &&
 		 (xmlStrEqual(nameURI, cur->nameURI)))) {
+		if ((cur->comp != NULL) &&
+		    (cur->comp->type == XSLT_FUNC_WITHPARAM))
+		    return(3);
+		if ((cur->comp != NULL) &&
+		    (cur->comp->type == XSLT_FUNC_PARAM))
+		    return(2);
 		return(1);
 	    }
 	}
@@ -334,8 +340,7 @@ xsltEvalVariable(xsltTransformContextPtr ctxt, xsltStackElemPtr elem,
 	if ((precomp == NULL) || (precomp->comp == NULL))
 	    xmlXPathFreeCompExpr(comp);
 	if (result == NULL) {
-	    xsltPrintErrorContext(ctxt, NULL, precomp->inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(ctxt, NULL, precomp->inst,
 		"Evaluating variable %s failed\n", elem->name);
 	    ctxt->state = XSLT_STATE_STOPPED;
 #ifdef WITH_XSLT_DEBUG_VARIABLE
@@ -357,16 +362,21 @@ xsltEvalVariable(xsltTransformContextPtr ctxt, xsltStackElemPtr elem,
 	     */
 	    xmlNodePtr container;
 	    xmlNodePtr oldInsert;
+	    xmlDocPtr  oldoutput;
 
 	    container = xmlNewDocNode(ctxt->document->doc, NULL,
 			      (const xmlChar *) "fake node libxslt", NULL);
 	    if (container == NULL)
 		return(NULL);
+	    container->parent = NULL;
 
+	    oldoutput = ctxt->output;
+	    ctxt->output = NULL;
 	    oldInsert = ctxt->insert;
 	    ctxt->insert = container;
 	    xsltApplyOneTemplate(ctxt, ctxt->node, elem->tree, NULL, NULL);
 	    ctxt->insert = oldInsert;
+	    ctxt->output = oldoutput;
 
 	    result = xmlXPathNewValueTree(container);
 	    if (result == NULL) {
@@ -460,8 +470,7 @@ xsltEvalGlobalVariable(xsltStackElemPtr elem, xsltTransformContextPtr ctxt) {
 	if ((precomp == NULL) || (precomp->comp == NULL))
 	    xmlXPathFreeCompExpr(comp);
 	if (result == NULL) {
-	    xsltPrintErrorContext(ctxt, NULL, precomp->inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(ctxt, NULL, precomp->inst,
 		"Evaluating global variable %s failed\n", elem->name);
 	    ctxt->state = XSLT_STATE_STOPPED;
 #ifdef WITH_XSLT_DEBUG_VARIABLE
@@ -483,16 +492,21 @@ xsltEvalGlobalVariable(xsltStackElemPtr elem, xsltTransformContextPtr ctxt) {
 	     */
 	    xmlNodePtr container;
 	    xmlNodePtr oldInsert;
+	    xmlDocPtr  oldoutput;
 
 	    container = xmlNewDocNode(ctxt->document->doc, NULL,
 			      (const xmlChar *) "fake node libxslt", NULL);
 	    if (container == NULL)
 		return(NULL);
+	    container->parent = NULL;
 
+	    oldoutput = ctxt->output;
+	    ctxt->output = NULL;
 	    oldInsert = ctxt->insert;
 	    ctxt->insert = container;
 	    xsltApplyOneTemplate(ctxt, ctxt->node, elem->tree, NULL, NULL);
 	    ctxt->insert = oldInsert;
+	    ctxt->output = oldoutput;
 
 	    result = xmlXPathNewValueTree(container);
 	    if (result == NULL) {
@@ -584,9 +598,9 @@ xsltEvalGlobalVariables(xsltTransformContextPtr ctxt) {
 		if ((elem->comp->inst != NULL) &&
 		    (def->comp != NULL) && (def->comp->inst != NULL) &&
 		    (elem->comp->inst->doc == def->comp->inst->doc)) {
-		    xsltPrintErrorContext(ctxt, style, elem->comp->inst);
-		    xsltGenericError(xsltGenericErrorContext,
+		    xsltTransformError(ctxt, style, elem->comp->inst,
 			"Global variable %s already defined\n", elem->name);
+		    style->errors++;
 		}
 	    }
 	    elem = elem->next;
@@ -658,8 +672,20 @@ xsltRegisterGlobalVariable(xsltStylesheetPtr style, const xmlChar *name,
 	elem->next = NULL;
 	style->variables = elem;
     } else {
-	while (tmp->next != NULL)
+	while (tmp != NULL) {
+	    if ((elem->comp->type == XSLT_FUNC_VARIABLE) &&
+		(tmp->comp->type == XSLT_FUNC_VARIABLE) &&
+		(xmlStrEqual(elem->name, tmp->name)) &&
+		((elem->nameURI == tmp->nameURI) ||
+		 (xmlStrEqual(elem->nameURI, tmp->nameURI)))) {
+		xsltTransformError(NULL, style, comp->inst,
+		"redefinition of global variable %s\n", elem->name);
+		style->errors++;
+	    }
+	    if (tmp->next == NULL)
+	        break;
 	    tmp = tmp->next;
+	}
 	elem->next = NULL;
 	tmp->next = elem;
     }
@@ -748,23 +774,27 @@ xsltProcessUserParamInternal(xsltTransformContextPtr ctxt,
 	    ns = xmlSearchNs(style->doc, xmlDocGetRootElement(style->doc),
 			     prefix);
 	    if (ns == NULL) {
-		xsltPrintErrorContext(ctxt, style, NULL);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(ctxt, style, NULL,
 		"user param : no namespace bound to prefix %s\n", prefix);
 		href = NULL;
 	    } else {
 		href = ns->href;
 	    }
 	    xmlFree(prefix);
+	    prefix = NULL;
 	} else {
 	    href = NULL;
 	}
 	xmlFree(ncname);
+	ncname = NULL;
     } else {
 	href = NULL;
 	ncname = xmlStrdup(name);
     }
 
+    if (ncname == NULL)
+	return (-1);
+
     /*
      * Do the evaluation if @eval is non-zero.
      */
@@ -794,8 +824,7 @@ xsltProcessUserParamInternal(xsltTransformContextPtr ctxt,
 	    xmlXPathFreeCompExpr(comp);
 	}
 	if (result == NULL) {
-	    xsltPrintErrorContext(ctxt, style, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(ctxt, style, NULL,
 		"Evaluating user parameter %s failed\n", name);
 	    ctxt->state = XSLT_STATE_STOPPED;
 	    xmlFree(ncname);
@@ -849,8 +878,7 @@ xsltProcessUserParamInternal(xsltTransformContextPtr ctxt,
     res = xmlHashAddEntry2(ctxt->globalVars, ncname, href, elem);
     if (res != 0) {
 	xsltFreeStackElem(elem);
-	xsltPrintErrorContext(ctxt, style, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, style, NULL,
 	    "Global parameter %s already defined\n", ncname);
     }
     xmlFree(ncname);
@@ -921,10 +949,9 @@ xsltQuoteUserParams(xsltTransformContextPtr ctxt, const char **params) {
 
 /**
  * xsltEvalOneUserParam:
- *
  * @ctxt:  the XSLT transformation context
  * @name:  a null terminated string giving the name of the parameter
- * @value  a null terminated string giving the XPath expression to be evaluated
+ * @value:  a null terminated string giving the XPath expression to be evaluated
  *
  * This is normally called from xsltEvalUserParams to process a single
  * parameter from a list of parameters.  The @value is evaluated as an
@@ -948,10 +975,9 @@ xsltEvalOneUserParam(xsltTransformContextPtr ctxt,
 
 /**
  * xsltQuoteOneUserParam:
- *
  * @ctxt:  the XSLT transformation context
  * @name:  a null terminated string giving the name of the parameter
- * @value  a null terminated string giving the parameter value
+ * @value:  a null terminated string giving the parameter value
  *
  * This is normally called from xsltQuoteUserParams to process a single
  * parameter from a list of parameters.  The @value is stored in the
@@ -1027,17 +1053,24 @@ static int
 xsltRegisterVariable(xsltTransformContextPtr ctxt, xsltStylePreCompPtr comp,
 		     xmlNodePtr tree, int param) {
     xsltStackElemPtr elem;
+    int present;
 
-    if (xsltCheckStackElem(ctxt, comp->name, comp->ns) != 0) {
-	if (!param) {
-	    xsltPrintErrorContext(ctxt, NULL, comp->inst);
-	    xsltGenericError(xsltGenericErrorContext,
-	    "xsl:variable : redefining %s\n", comp->name);
+    present = xsltCheckStackElem(ctxt, comp->name, comp->ns);
+    if (param == 0) {
+	if ((present != 0) && (present != 3)) {
+	    xsltTransformError(ctxt, NULL, comp->inst,
+		"xsl:variable : redefining %s\n", comp->name);
+	    return(0);
+	}
+    } else if (present != 0) {
+	if ((present == 1) || (present == 2)) {
+	    xsltTransformError(ctxt, NULL, comp->inst,
+		"xsl:param : redefining %s\n", comp->name);
+	    return(0);
 	}
 #ifdef WITH_XSLT_DEBUG_VARIABLE
-	else
-	    xsltGenericDebug(xsltGenericDebugContext,
-		     "param %s defined by caller\n", comp->name);
+	xsltGenericDebug(xsltGenericDebugContext,
+		 "param %s defined by caller\n", comp->name);
 #endif
 	return(0);
     }
@@ -1132,7 +1165,7 @@ xsltVariableLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
  * parse an XSLT transformation param declaration, compute
  * its value but doesn't record it.
  *
- * It returns the new xsltStackElemPtr or NULL
+ * Returns the new xsltStackElemPtr or NULL
  */
 
 xsltStackElemPtr
@@ -1145,15 +1178,13 @@ xsltParseStylesheetCallerParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 	return(NULL);
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:param : compilation error\n");
 	return(NULL);
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:param : missing name attribute\n");
 	return(NULL);
     }
@@ -1197,15 +1228,13 @@ xsltParseGlobalVariable(xsltStylesheetPtr style, xmlNodePtr cur) {
     xsltStylePreCompute(style, cur);
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	     "xsl:variable : compilation failed\n");
 	return;
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:variable : missing name attribute\n");
 	return;
     }
@@ -1238,15 +1267,13 @@ xsltParseGlobalParam(xsltStylesheetPtr style, xmlNodePtr cur) {
     xsltStylePreCompute(style, cur);
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	     "xsl:param : compilation failed\n");
 	return;
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:param : missing name attribute\n");
 	return;
     }
@@ -1278,15 +1305,13 @@ xsltParseStylesheetVariable(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	     "xsl:variable : compilation failed\n");
 	return;
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:variable : missing name attribute\n");
 	return;
     }
@@ -1317,15 +1342,13 @@ xsltParseStylesheetParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	     "xsl:param : compilation failed\n");
 	return;
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:param : missing name attribute\n");
 	return;
     }
@@ -1378,8 +1401,7 @@ xsltXPathVariableLookup(void *ctxt, const xmlChar *name,
     context = (xsltTransformContextPtr) ctxt;
     ret = xsltVariableLookup(context, name, ns_uri);
     if (ret == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, NULL,
 	    "unregistered variable %s\n", name);
     }
 #ifdef WITH_XSLT_DEBUG_VARIABLE
