@@ -40,6 +40,7 @@
 #include "extensions.h"
 #include "preproc.h"
 #include "extra.h"
+#include "security.h"
 
 #ifdef WITH_XSLT_DEBUG
 #define WITH_XSLT_DEBUG_PARSING
@@ -67,53 +68,67 @@ double xmlXPathStringEvalNumber(const xmlChar *str);
 #define IS_BLANK_NODE(n)						\
     (((n)->type == XML_TEXT_NODE) && (xsltIsBlank((n)->content)))
 
-/*
- * Generic function for accessing stacks in the stylesheet
+/**
+ * exclPrefixPush:
+ * @style: the transformation stylesheet
+ * @value:  the excluded prefix to push on the stack
+ *
+ * Push an excluded prefix on the stack
+ *
+ * Returns the new index in the stack or 0 in case of error
  */
-
-#define PUSH_AND_POP(scope, type, name)					\
-scope int name##Push(xsltStylesheetPtr style, type value)	 {	\
-    if (style->name##Max == 0) {					\
-	style->name##Max = 4;						\
-        style->name##Tab = (type *) xmlMalloc(style->name##Max *	\
-	              sizeof(style->name##Tab[0]));			\
-        if (style->name##Tab == NULL) {					\
-	    xmlGenericError(xmlGenericErrorContext,			\
-		    "malloc failed !\n");				\
-	    return(0);							\
-	}								\
-    }									\
-    if (style->name##Nr >= style->name##Max) {				\
-	style->name##Max *= 2;						\
-        style->name##Tab = (type *) xmlRealloc(style->name##Tab,	\
-	             style->name##Max * sizeof(style->name##Tab[0]));	\
-        if (style->name##Tab == NULL) {					\
-	    xmlGenericError(xmlGenericErrorContext,			\
-		    "realloc failed !\n");				\
-	    return(0);							\
-	}								\
-    }									\
-    style->name##Tab[style->name##Nr] = value;				\
-    style->name = value;						\
-    return(style->name##Nr++);						\
-}									\
-scope type name##Pop(xsltStylesheetPtr style)	 {			\
-    type ret;								\
-    if (style->name##Nr <= 0) return(0);				\
-    style->name##Nr--;							\
-    if (style->name##Nr > 0)						\
-	style->name = style->name##Tab[style->name##Nr - 1];		\
-    else								\
-        style->name = NULL;						\
-    ret = style->name##Tab[style->name##Nr];				\
-    style->name##Tab[style->name##Nr] = 0;				\
-    return(ret);							\
-}									\
-
-/*
- * Those macros actually generate the functions
+static int
+exclPrefixPush(xsltStylesheetPtr style, xmlChar * value)
+{
+    if (style->exclPrefixMax == 0) {
+        style->exclPrefixMax = 4;
+        style->exclPrefixTab =
+            (xmlChar * *)xmlMalloc(style->exclPrefixMax *
+                                   sizeof(style->exclPrefixTab[0]));
+        if (style->exclPrefixTab == NULL) {
+            xmlGenericError(xmlGenericErrorContext, "malloc failed !\n");
+            return (0);
+        }
+    }
+    if (style->exclPrefixNr >= style->exclPrefixMax) {
+        style->exclPrefixMax *= 2;
+        style->exclPrefixTab =
+            (xmlChar * *)xmlRealloc(style->exclPrefixTab,
+                                    style->exclPrefixMax *
+                                    sizeof(style->exclPrefixTab[0]));
+        if (style->exclPrefixTab == NULL) {
+            xmlGenericError(xmlGenericErrorContext, "realloc failed !\n");
+            return (0);
+        }
+    }
+    style->exclPrefixTab[style->exclPrefixNr] = value;
+    style->exclPrefix = value;
+    return (style->exclPrefixNr++);
+}
+/**
+ * exclPrefixPop:
+ * @style: the transformation stylesheet
+ *
+ * Pop an excluded prefix value from the stack
+ *
+ * Returns the stored excluded prefix value
  */
-PUSH_AND_POP(static, xmlChar *, exclPrefix)
+static xmlChar *
+exclPrefixPop(xsltStylesheetPtr style)
+{
+    xmlChar *ret;
+
+    if (style->exclPrefixNr <= 0)
+        return (0);
+    style->exclPrefixNr--;
+    if (style->exclPrefixNr > 0)
+        style->exclPrefix = style->exclPrefixTab[style->exclPrefixNr - 1];
+    else
+        style->exclPrefix = NULL;
+    ret = style->exclPrefixTab[style->exclPrefixNr];
+    style->exclPrefixTab[style->exclPrefixNr] = 0;
+    return (ret);
+}
 
 /************************************************************************
  *									*
@@ -239,11 +254,13 @@ xsltFreeDecimalFormatList(xsltStylesheetPtr self)
  * @name: the decimal-format name to find
  *
  * Find decimal-format by name
+ *
+ * Returns the xsltDecimalFormatPtr
  */
 xsltDecimalFormatPtr
 xsltDecimalFormatGetByName(xsltStylesheetPtr sheet, xmlChar *name)
 {
-    xsltDecimalFormatPtr result;
+    xsltDecimalFormatPtr result = NULL;
 
     if (name == NULL)
 	return sheet->decimalFormat;
@@ -271,8 +288,7 @@ xsltNewTemplate(void) {
 
     cur = (xsltTemplatePtr) xmlMalloc(sizeof(xsltTemplate));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltNewTemplate : malloc failed\n");
 	return(NULL);
     }
@@ -331,8 +347,7 @@ xsltNewStylesheet(void) {
 
     cur = (xsltStylesheetPtr) xmlMalloc(sizeof(xsltStylesheet));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltNewStylesheet : malloc failed\n");
 	return(NULL);
     }
@@ -397,6 +412,7 @@ xsltAllocateExtraCtxt(xsltTransformContextPtr ctxt)
 	    for (i = 0;i < ctxt->extrasMax;i++) {
 		ctxt->extras[i].info = NULL;
 		ctxt->extras[i].deallocate = NULL;
+		ctxt->extras[i].val = NULL;
 	    }
 
 	} else {
@@ -415,6 +431,7 @@ xsltAllocateExtraCtxt(xsltTransformContextPtr ctxt)
 	    for (i = ctxt->extrasNr;i < ctxt->extrasMax;i++) {
 		ctxt->extras[i].info = NULL;
 		ctxt->extras[i].deallocate = NULL;
+		ctxt->extras[i].val = NULL;
 	    }
 	}
     }
@@ -642,8 +659,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
                 (xmlStrEqual(prop, (const xmlChar *) "text"))) {
                 style->method = prop;
             } else {
-		xsltPrintErrorContext(NULL, style, cur);
-                xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, style, cur,
                                  "invalid value for method: %s\n", prop);
                 style->warnings++;
             }
@@ -679,8 +695,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         } else if (xmlStrEqual(prop, (const xmlChar *) "no")) {
             style->standalone = 0;
         } else {
-	    xsltPrintErrorContext(NULL, style, cur);
-            xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
                              "invalid value for standalone: %s\n", prop);
             style->warnings++;
         }
@@ -694,8 +709,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         } else if (xmlStrEqual(prop, (const xmlChar *) "no")) {
             style->indent = 0;
         } else {
-	    xsltPrintErrorContext(NULL, style, cur);
-            xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
                              "invalid value for indent: %s\n", prop);
             style->warnings++;
         }
@@ -710,8 +724,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         } else if (xmlStrEqual(prop, (const xmlChar *) "no")) {
             style->omitXmlDeclaration = 0;
         } else {
-	    xsltPrintErrorContext(NULL, style, cur);
-            xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
                              "invalid value for omit-xml-declaration: %s\n",
                              prop);
             style->warnings++;
@@ -759,6 +772,13 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         }
         xmlFree(elements);
     }
+
+    prop = xsltGetNsProp(cur, (const xmlChar *) "media-type", XSLT_NAMESPACE);
+    if (prop != NULL) {
+	if (style->mediaType)
+	    xmlFree(style->mediaType);
+	style->mediaType = prop;
+    }
 }
 
 /**
@@ -785,16 +805,14 @@ xsltParseStylesheetDecimalFormat(xsltStylesheetPtr style, xmlNodePtr cur)
     if (prop != NULL) {
 	format = xsltDecimalFormatGetByName(style, prop);
 	if (format != NULL) {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 	 "xsltParseStylestyleDecimalFormat: %s already exists\n", prop);
 	    style->warnings++;
 	    return;
 	}
 	format = xsltNewDecimalFormat(prop);
 	if (format == NULL) {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
      "xsltParseStylestyleDecimalFormat: failed creating new decimal-format\n");
 	    style->errors++;
 	    return;
@@ -889,8 +907,7 @@ xsltParseStylesheetPreserveSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
 
     elements = xsltGetNsProp(cur, (const xmlChar *)"elements", XSLT_NAMESPACE);
     if (elements == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsltParseStylesheetPreserveSpace: missing elements attribute\n");
 	style->warnings++;
 	return;
@@ -970,8 +987,7 @@ xsltParseStylesheetExtPrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    else
 		ns = xmlSearchNs(style->doc, cur, prefix);
 	    if (ns == NULL) {
-		xsltPrintErrorContext(NULL, style, cur);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, style, cur,
 	    "xsl:extension-element-prefix : undefined namespace %s\n",
 	                         prefix);
 		style->warnings++;
@@ -1008,8 +1024,7 @@ xsltParseStylesheetStripSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
 
     elements = xsltGetNsProp(cur, (const xmlChar *)"elements", XSLT_NAMESPACE);
     if (elements == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsltParseStylesheetStripSpace: missing elements attribute\n");
 	style->warnings++;
 	return;
@@ -1092,8 +1107,7 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    else
 		ns = xmlSearchNs(style->doc, cur, prefix);
 	    if (ns == NULL) {
-		xsltPrintErrorContext(NULL, style, cur);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, style, cur,
 	    "xsl:exclude-result-prefixes : undefined namespace %s\n",
 	                         prefix);
 		style->warnings++;
@@ -1286,8 +1300,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 		    if (style->nsHash == NULL) {
 			style->nsHash = xmlHashCreate(10);
 			if (style->nsHash == NULL) {
-			    xsltPrintErrorContext(NULL, style, cur);
-			    xsltGenericError(xsltGenericErrorContext,
+			    xsltTransformError(NULL, style, cur,
 		 "xsltGatherNamespaces: failed to create hash table\n");
 			    style->errors++;
 			    return;
@@ -1295,8 +1308,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 		    }
 		    URI = xmlHashLookup(style->nsHash, ns->prefix);
 		    if ((URI != NULL) && (!xmlStrEqual(URI, ns->href))) {
-			xsltPrintErrorContext(NULL, style, cur);
-			xsltGenericError(xsltGenericErrorContext,
+			xsltTransformError(NULL, style, cur,
 	     "Namespaces prefix %s used for multiple namespaces\n");
 			style->warnings++;
 		    } else if (URI == NULL) {
@@ -1392,8 +1404,7 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 			    noesc = 1;
 			} else if (!xmlStrEqual(prop,
 						(const xmlChar *)"no")){
-			    xsltPrintErrorContext(NULL, style, cur);
-			    xsltGenericError(xsltGenericErrorContext,
+			    xsltTransformError(NULL, style, cur,
 	     "xsl:text: disable-output-escaping allows only yes or no\n");
 			    style->warnings++;
 
@@ -1402,10 +1413,13 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 		    }
 
 		    while (text != NULL) {
+			if (text->type == XML_COMMENT_NODE) {
+			    text = text->next;
+			    continue;
+			}
 			if ((text->type != XML_TEXT_NODE) &&
 			     (text->type != XML_CDATA_SECTION_NODE)) {
-			    xsltPrintErrorContext(NULL, style, cur);
-			    xsltGenericError(xsltGenericErrorContext,
+			    xsltTransformError(NULL, style, cur,
 		 "xsltParseTemplateContent: xslt:text content problem\n");
 			    style->errors++;
 			    break;
@@ -1497,8 +1511,7 @@ skip_children:
 	    xmlNodePtr param = cur;
 
             cur = cur->next;
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 		"xsltParseTemplateContent: ignoring misplaced param element\n");
 	    style->warnings++;
 	    xmlUnlinkNode(param);
@@ -1550,8 +1563,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 	     "xsltParseStylesheetKey: name %s\n", name);
 #endif
     } else {
-	xsltPrintErrorContext(NULL, style, key);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing name\n");
 	style->errors++;
 	goto error;
@@ -1559,8 +1571,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 
     match = xsltGetNsProp(key, (const xmlChar *)"match", XSLT_NAMESPACE);
     if (match == NULL) {
-	xsltPrintErrorContext(NULL, style, key);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing match\n");
 	style->errors++;
 	goto error;
@@ -1568,8 +1579,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 
     use = xsltGetNsProp(key, (const xmlChar *)"use", XSLT_NAMESPACE);
     if (use == NULL) {
-	xsltPrintErrorContext(NULL, style, key);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing use\n");
 	style->errors++;
 	goto error;
@@ -1727,15 +1737,13 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 
     prop = xsltGetNsProp(top, (const xmlChar *)"version", XSLT_NAMESPACE);
     if (prop == NULL) {
-	xsltPrintErrorContext(NULL, style, top);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, top,
 	    "xsl:version is missing: document may not be a stylesheet\n");
 	style->warnings++;
     } else {
 	if ((!xmlStrEqual(prop, (const xmlChar *)"1.0")) &&
             (!xmlStrEqual(prop, (const xmlChar *)"1.1"))) {
-	    xsltPrintErrorContext(NULL, style, top);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, top,
 		"xsl:version: only 1.0 features are supported\n");
 	     /* TODO set up compatibility when not XSLT 1.0 */
 	    style->warnings++;
@@ -1750,18 +1758,19 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
     /*
      * process xsl:import elements
      */
-	while (cur != NULL) {
-		if (IS_BLANK_NODE(cur)) {
-			cur = cur->next;
-			continue;
-		}
-		if (IS_XSLT_ELEM(cur) && IS_XSLT_NAME(cur, "import")) {
-			if (xsltParseStylesheetImport(style, cur) != 0)
-				style->errors++;
-		} else
-			break;
-		cur = cur->next;
-	}
+    while (cur != NULL) {
+	    if (IS_BLANK_NODE(cur)) {
+		    cur = cur->next;
+		    continue;
+	    }
+	    if (IS_XSLT_ELEM(cur) && IS_XSLT_NAME(cur, "import")) {
+		    if (xsltParseStylesheetImport(style, cur) != 0)
+			    style->errors++;
+	    } else
+		    break;
+	    cur = cur->next;
+    }
+
     /*
      * process other top-level elements
      */
@@ -1772,8 +1781,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	}
 	if (cur->type == XML_TEXT_NODE) {
 	    if (cur->content != NULL) {
-		xsltPrintErrorContext(NULL, style, cur);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, style, cur,
 		    "misplaced text element: '%s'\n", cur->content);
 	    }
 	    style->errors++;
@@ -1805,8 +1813,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	    continue;
 	}
 	if (IS_XSLT_NAME(cur, "import")) {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 			"xsltParseStylesheetTop: ignoring misplaced import element\n");
 	    style->errors++;
     } else if (IS_XSLT_NAME(cur, "include")) {
@@ -1836,8 +1843,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
     } else if (IS_XSLT_NAME(cur, "namespace-alias")) {
 	    xsltNamespaceAlias(style, cur);
 	} else {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 			"xsltParseStylesheetTop: ignoring unknown %s element\n",
 			cur->name);
 	    style->warnings++;
@@ -1877,8 +1883,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
      */
     cur = xmlDocGetRootElement(doc);
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, ret, (xmlNodePtr) doc);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, ret, (xmlNodePtr) doc,
 		"xsltParseStylesheetProcess : empty stylesheet\n");
 	return(NULL);
     }
@@ -1903,8 +1908,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	 */
 	prop = xsltGetNsProp(cur, (const xmlChar *)"version", XSLT_NAMESPACE);
 	if (prop == NULL) {
-	    xsltPrintErrorContext(NULL, ret, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, ret, cur,
 		"xsltParseStylesheetProcess : document is not a stylesheet\n");
 	    return(NULL);
 	}
@@ -1915,8 +1919,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 #endif
 	
 	if (!xmlStrEqual(prop, (const xmlChar *)"1.0")) {
-	    xsltPrintErrorContext(NULL, ret, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, ret, cur,
 		"xsl:version: only 1.0 features are supported\n");
 	     /* TODO set up compatibility when not XSLT 1.0 */
 	    ret->warnings++;
@@ -1942,7 +1945,38 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	template->content = doc->children;
 	xsltAddTemplate(ret, template, NULL, NULL);
     }
-    xsltResolveStylesheetAttributeSet(ret);
+
+    return(ret);
+}
+
+/**
+ * xsltParseStylesheetImportedDoc:
+ * @doc:  and xmlDoc parsed XML
+ *
+ * parse an XSLT stylesheet building the associated structures
+ * except the processing not needed for imported documents.
+ *
+ * Returns a new XSLT stylesheet structure.
+ */
+
+xsltStylesheetPtr
+xsltParseStylesheetImportedDoc(xmlDocPtr doc) {
+    xsltStylesheetPtr ret;
+
+    if (doc == NULL)
+	return(NULL);
+
+    ret = xsltNewStylesheet();
+    if (ret == NULL)
+	return(NULL);
+    
+    ret->doc = doc;
+    xsltGatherNamespaces(ret);
+    if (xsltParseStylesheetProcess(ret, doc) == NULL) {
+	ret->doc = NULL;
+	xsltFreeStylesheet(ret);
+	ret = NULL;
+    }
 
     return(ret);
 }
@@ -1960,20 +1994,11 @@ xsltStylesheetPtr
 xsltParseStylesheetDoc(xmlDocPtr doc) {
     xsltStylesheetPtr ret;
 
-    if (doc == NULL)
-	return(NULL);
-
-    ret = xsltNewStylesheet();
+    ret = xsltParseStylesheetImportedDoc(doc);
     if (ret == NULL)
 	return(NULL);
-    
-    ret->doc = doc;
-    xsltGatherNamespaces(ret);
-	if (xsltParseStylesheetProcess(ret, doc) == NULL) {
-		ret->doc = NULL;
-		xsltFreeStylesheet(ret);
-		ret = NULL;
-	}
+
+    xsltResolveStylesheetAttributeSet(ret);
 
     return(ret);
 }
@@ -1989,6 +2014,7 @@ xsltParseStylesheetDoc(xmlDocPtr doc) {
 
 xsltStylesheetPtr
 xsltParseStylesheetFile(const xmlChar* filename) {
+    xsltSecurityPrefsPtr sec;
     xsltStylesheetPtr ret;
     xmlDocPtr doc;
     
@@ -2001,10 +2027,25 @@ xsltParseStylesheetFile(const xmlChar* filename) {
 	    "xsltParseStylesheetFile : parse %s\n", filename);
 #endif
 
+    /*
+     * Security framework check
+     */
+    sec = xsltGetDefaultSecurityPrefs();
+    if (sec != NULL) {
+	int res;
+
+	res = xsltCheckRead(sec, NULL, filename);
+	if (res == 0) {
+	    xsltTransformError(NULL, NULL, NULL,
+		 "xsltParseStylesheetFile: read rights for %s denied\n",
+			     filename);
+	    return(NULL);
+	}
+    }
+
     doc = xmlParseFile((const char *) filename);
     if (doc == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltParseStylesheetFile : cannot parse %s\n", filename);
 	return(NULL);
     }
@@ -2164,8 +2205,7 @@ xsltLoadStylesheetPI(xmlDocPtr doc) {
 #endif
 	URI = xmlParseURI((const char *) href);
 	if (URI == NULL) {
-	    xsltPrintErrorContext(NULL, NULL, child);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, child,
 		    "xml-stylesheet : href %s is not valid\n", href);
 	    xmlFree(href);
 	    return(NULL);
@@ -2185,8 +2225,7 @@ xsltLoadStylesheetPI(xmlDocPtr doc) {
 	    else
 		ID = xmlGetID(doc, (const xmlChar *) URI->fragment);
 	    if (ID == NULL) {
-		xsltPrintErrorContext(NULL, NULL, child);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, NULL, child,
 		    "xml-stylesheet : no ID %s found\n", URI->fragment);
 	    } else {
 		xmlDocPtr fake;
